from functools import partial

class ByteConverter:
    def __init__(self):
        self.set_int_conversion()
        self.set_hex_conversion()
        self.set_str_conversion()
        
    def set_hex_conversion(self, pad_with_zero:bool=True, capitalize:bool=True,
                           default_bytesize:int=8):
        self._hex_pad_with_zero = pad_with_zero
        self._hex_capitalize = capitalize
        self._hex_default_bytesize = default_bytesize
        hex_format = "X" if self._hex_capitalize else "x"
        if self._hex_pad_with_zero:
            format_str = "{{int_value:0{{bytesize}}{hex_format}}}".format(hex_format=hex_format)
        else:
            format_str = "{{int_value:{hex_format}}}".format(hex_format=hex_format)
        self._hex_converter = partial(format_str.format, bytesize=self._hex_default_bytesize)
        
    def set_str_conversion(self, codec:str="mbcs"):
        self._str_codec = codec
        self._str_converter = lambda x: x.decode(self._str_codec)
    
    def set_int_conversion(self, byteorder:str="little"):
        self._byteorder = byteorder
        self._int_converter = partial(int.from_bytes, byteorder=self._byteorder)
    
    def to_int(self, byte_value):
        return self._int_converter(byte_value)
    
    def to_hex(self, byte_value, bytesize:int=8):
        int_value = self.to_int(byte_value)
        return self._hex_converter(int_value=int_value, bytesize=bytesize)
    
    def to_str(self, byte_value):
        return self._str_converter(byte_value)
    
    def int2hex(self, int_value:int, bytesize:int=8):
        return self._hex_converter(int_value=int_value, bytesize=bytesize)


from typing import Optional

from ctypes import *
from ctypes.wintypes import *

OpenProcess = windll.kernel32.OpenProcess
ReadProcessMemory = windll.kernel32.ReadProcessMemory
CloseHandle = windll.kernel32.CloseHandle

from dragonsoft.converters import ByteConverter

class ProcessMemoryScanner:
    def __init__(self, PID:int):
        self.processHandle = None
        self.open_process(PID)
        self.converter = ByteConverter()
    
    def open_process(self, PID:int):
        if self.processHandle is not None:
            self.close_handle()
        PROCESS_ALL_ACCESS = 0x1F0FFF
        processHandle = OpenProcess(PROCESS_ALL_ACCESS, False, PID)
        self.processHandle = processHandle
    
    def close_handle(self):
        if self.processHandle is None:
            return None
        CloseHandle(self.processHandle)
        self.processHandle = None
        
    def get_bytes(self, lpBaseAddress:int, nSize:int=4):
        lpBuffer            = ctypes.create_string_buffer(nSize)
        lpNumberOfBytesRead = c_ulong(0)
        success = ReadProcessMemory(self.processHandle, lpBaseAddress, lpBuffer, nSize, byref(lpNumberOfBytesRead))
        if success:
            return lpBuffer.value
        else:
            print(f'ERROR: Failed to read memory from address "{self.converter.int2hex(lpBaseAddress)}"')
            return None
        
    def get_value(self, lpBaseAddress:int, nSize:int=4):
        lpPointer           = c_void_p()
        lpBuffer            = byref(lpPointer)
        lpNumberOfBytesRead = c_ulong(0)
        success = ReadProcessMemory(self.processHandle, lpBaseAddress, lpBuffer, nSize, byref(lpNumberOfBytesRead))
        if success:
            if lpPointer.value is None:
                return 0
            return lpPointer.value
        else:
            print(f'ERROR: Failed to read memory from address "{self.converter.int2hex(lpBaseAddress)}"')
            return None
    
    def get_zero_terminated_bytes(self, lpBaseAddress:int, maxSize:int=36):
        byte_value = b""
        for i in range(maxSize):
            value = self.get_bytes(lpBaseAddress + i, nSize=1)
            if not value:
                break
            byte_value += value
        return byte_value
        
    def get_int(self, lpBaseAddress:int, nSize:int=4):
        return self.get_value(lpBaseAddress, nSize=nSize)
    
    def get_hex(self, lpBaseAddress:int, nSize:int=4):
        value = self.get_value(lpBaseAddress, nSize=nSize)
        if value is not None:
            return self.converter.int2hex(value)
        return None
    
    def get_str(self, lpBaseAddress:int, maxSize:int=36):
        byte_value = self.get_zero_terminated_bytes(lpBaseAddress, maxSize=maxSize)
        return self.converter.to_str(byte_value)


import os
import re
import sys
import glob
import builtins
from contextlib import contextmanager

import setuptools
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext as _build_ext
from setuptools.command.install import install as _install

with open("README.md", "r") as fh:
    long_description = fh.read()
    
PACKAGENAME = "dragonsoft"
    
VERSIONFILE = f"{PACKAGENAME}/_version.py"

verstrline  = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))    

setup(
    name=PACKAGENAME,
    version=verstr,
    author="Alkaid",
    author_email="",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          #'click'
      ],
    #scripts=[f'bin/{PACKAGENAME}'],
    python_requires='>=3.6',
)
